/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.layer0.request;

import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.common.exception.DebugException;
import org.simantics.db.common.request.TernaryRead;
import org.simantics.db.exception.AdaptionException;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.ResourceNotFoundException;
import org.simantics.db.layer0.adapter.Valuations;
import org.simantics.layer0.Layer0;
import org.simantics.scl.reflection.OntologyVersions;
import org.simantics.simulation.ontology.SimulationResource;

public class PossibleSessionValuation extends TernaryRead<Resource, String, String, Resource> {

    public PossibleSessionValuation(Resource resource, String identifier) {
        super(resource, identifier, "");
    }

    public PossibleSessionValuation(Resource resource, String identifier, String suffix) {
        super(resource, identifier, suffix);
    }

    @Override
    public Resource perform(ReadGraph graph) throws DatabaseException {
        try {
            Layer0 l0 = Layer0.getInstance(graph);
            String valuationName = null;
            Valuations valuations = graph.adapt(parameter, Valuations.class);

            String uri = OntologyVersions.getInstance().currentVersion("http://www.simantics.org/Modeling-0.0/CompositeToDiagram");
            Resource map = graph.getResource(uri);

            Resource diagram = graph.syncRequest(new PossibleDiagram(parameter));
            if(diagram == null) {
                Resource d = parameter;
                while((d = graph.getPossibleObject(d, l0.PartOf)) != null) {
                    diagram = graph.getPossibleObject(d, map);
                    if(diagram != null) break;
                }
            }

            if(diagram != null) {
                SimulationResource SIMU = SimulationResource.getInstance(graph);
                for(Resource run : graph.getObjects(diagram, l0.ConsistsOf)) {
                    String name = graph.getPossibleRelatedValue(run, l0.HasName, Bindings.STRING);
                    if(parameter2.equals(name)) {
                        valuationName = graph.getPossibleRelatedValue(run, SIMU.HasActiveExperiment);
                        break;
                    }
                }
            }

            Resource value = valuations.getValue(graph, null, valuationName, parameter3);
            return graph.getSingleObject(value, l0.ConsistsOf);

        } catch (ResourceNotFoundException e) {
            if (DebugException.DEBUG)
                e.printStackTrace();
            return null;
        } catch (AdaptionException e) {
            if (DebugException.DEBUG)
                e.printStackTrace();
            return null;
        }

    }

}
